package ru.landgrafhomyak.db.skeleton1.api.errors

import kotlin.jvm.JvmStatic
import kotlinx.coroutines.CopyableThrowable
import kotlinx.coroutines.ExperimentalCoroutinesApi
import ru.landgrafhomyak.db.skeleton1.api.ReflectionApi
import ru.landgrafhomyak.db.skeleton1.api.statement.InputParam
import ru.landgrafhomyak.db.skeleton1.api.statement._Statement

@ReflectionApi
@OptIn(ExperimentalCoroutinesApi::class)
public class EntityDeletedException : IllegalStateException, CopyableThrowable<EntityDeletedException> {
	public val entity: Any

	public constructor(entity: Any) : super(formatMessage(entity)) {
		this.entity = entity
	}

	public constructor(entity: Any, customMessage: String) : super(customMessage) {
		this.entity = entity
	}

	public constructor(entity: Any, customMessage: String, cause: Throwable) : super(customMessage, cause) {
		this.entity = entity
	}

	public constructor(entity: Any, cause: Throwable) : super(formatMessage(entity), cause) {
		this.entity = entity
	}

	public constructor(
		entity: Any,
		message: String?,
		cause: Throwable?,
		@Suppress("unused", "LocalVariableName") _marker: Unit
	) : super(message, cause) {
		this.entity = entity
	}


	override fun createCopy(): EntityDeletedException? {
		return EntityDeletedException(
			entity = this.entity,
			message = this.message,
			cause = this.cause,
			_marker = Unit
		)
	}

	public companion object {
		@JvmStatic
		public fun formatMessage(entity: Any): String =
			"Attempt to access deleted $entity"
	}
}